"""Canonical teleprompt import surface for Synth DSPy compatibility."""

from __future__ import annotations

from synth_ai.dspy.gepa import GEPA
from synth_ai.dspy.miprov2 import MIPROv2

__all__ = ["GEPA", "MIPROv2"]
